@@ -2,6 +2,7 @@ import json
 from functools import lru_cache
 
 import dateutil.parser
+from django.conf import settings
 from django.utils import dateparse, html, timezone
 
 from .common import all_subclasses, get_optimal_decimal_places
@@ -41,9 +42,9 @@ class BaseType(metaclass=TypeMeta):
         return {}
 
     @staticmethod
-    def format(value, choices=None):
+    def get_formatter(choices):
         assert not choices
-        return value
+        return lambda value: value
 
     @staticmethod
     def _parse(value):
@@ -91,9 +92,11 @@ class ChoiceTypeMixin:
     default_value = None
 
     @staticmethod
-    def format(value, choices=None):
+    def get_formatter(choices):
         assert choices
-        return dict(choices)[value]
+        choices = dict(choices)
+        choices[None] = None
+        return lambda value: choices.get(value, value)
 
 
 class StringChoiceType(ChoiceTypeMixin, BaseType):
@@ -110,10 +113,16 @@ class ArrayTypeMixin:
     default_value = None
 
     @staticmethod
-    def format(value, choices=None):  # pragma: postgres
+    def get_formatter(choices):  # pragma: postgres
         if choices:
-            value = [dict(choices)[v] if v is not None else None for v in value]
-        return ", ".join(str(v) for v in value)
+            choices = dict(choices)
+            choices[None] = None
+            return (
+                lambda value: None
+                if value is None
+                else ", ".join(str(choices.get(v, v)) for v in value)
+            )
+        return lambda value: None if value is None else ", ".join(str(v) for v in value)
 
 
 class StringArrayType(ArrayTypeMixin, BaseType):
@@ -161,9 +170,9 @@ class NumberType(BaseType):
         }
 
     @staticmethod
-    def format(value, choices=None):
+    def get_formatter(choices):
         assert not choices
-        return float(value)
+        return lambda value: None if value is None else float(value)
 
     @staticmethod
     def _parse(value):
@@ -254,9 +263,9 @@ class DurationType(BaseType):
         return res
 
     @staticmethod
-    def format(value, choices=None):
+    def get_formatter(choices):
         assert not choices
-        return str(value)
+        return lambda value: None if value is None else str(value)
 
 
 class DateTimeType(BaseType):
@@ -282,11 +291,14 @@ class DateTimeType(BaseType):
         return timezone.make_aware(dateutil.parser.parse(value))
 
     @staticmethod
-    def format(value, choices=None):
+    def get_formatter(choices):
         assert not choices
-        if not timezone.is_naive(value):
-            value = timezone.make_naive(value)
-        return str(value)
+        if settings.USE_TZ:
+            return (
+                lambda value: None if value is None else str(timezone.make_naive(value))
+            )
+        else:
+            return lambda value: None if value is None else str(value)
 
 
 class DateType(BaseType):
@@ -312,9 +324,9 @@ class DateType(BaseType):
         return timezone.make_aware(dateutil.parser.parse(value)).date()
 
     @staticmethod
-    def format(value, choices=None):
+    def get_formatter(choices):
         assert not choices
-        return str(value)
+        return lambda value: None if value is None else str(value)
 
 
 class WeekDayType(BaseType):
@@ -336,9 +348,9 @@ class WeekDayType(BaseType):
     ]
 
     @classmethod
-    def format(cls, value, choices=None):
+    def get_formatter(cls, value, choices=None):
         assert not choices
-        return cls._days[value - 1]
+        return lambda value: None if value is None else cls._days[value - 1]
 
     @classmethod
     def _parse(cls, value):
@@ -372,9 +384,9 @@ class MonthType(BaseType):
     ]
 
     @classmethod
-    def format(cls, value, choices=None):
+    def get_formatter(cls, value, choices=None):
         assert not choices
-        return cls._months[value - 1]
+        return lambda value: None if value is None else cls._months[value - 1]
 
     @classmethod
     def _parse(cls, value):
@@ -385,9 +397,10 @@ class MonthType(BaseType):
 
 
 class HTMLType(StringType):
-    @classmethod
-    def format(cls, value, choices=None):
-        return html.conditional_escape(value)
+    @staticmethod
+    def get_formatter(value, choices=None):
+        assert not choices
+        return lambda value: None if value is None else html.conditional_escape(value)
 
 
 class BooleanType(BaseType):
@@ -411,10 +424,10 @@ class BooleanType(BaseType):
         else:
             raise ValueError("Expected 'true' or 'false'")
 
-    @classmethod
-    def format(cls, value, choices=None):
+    @staticmethod
+    def get_formatter(value, choices=None):
         assert not choices
-        return bool(value)
+        return lambda value: None if value is None else bool(value)
 
 
 class IsNullType(BooleanType):
@@ -425,16 +438,16 @@ class IsNullType(BooleanType):
         return {"equals": BooleanType}
 
     @staticmethod
-    def format(value, choices=None):
+    def get_formatter(choices):
         assert not choices
-        return "IsNull" if value else "NotNull"
+        return lambda value: None if value is None else "IsNull" if value else "NotNull"
 
 
 class UnknownType(BaseType):
     @staticmethod
-    def format(value, choices=None):
+    def get_formatter(choices):
         assert not choices
-        return str(value)
+        return lambda value: None if value is None else str(value)
 
     @staticmethod
     def _lookups():
