@@ -2,7 +2,7 @@ from django.contrib import admin
 from django.contrib.contenttypes.admin import GenericInlineModelAdmin
 from django.db.models import F
 
-from data_browser.helpers import AdminMixin, annotation, ddb_hide
+from data_browser.helpers import AdminMixin, annotation, attributes
 
 from . import models
 
@@ -105,7 +105,7 @@ class ProductMixin:
     readonly_fields = ["id", "is_onsale", "hidden_calculated"]
     ddb_default_filters = "default=filter&filter=default"
 
-    @ddb_hide
+    @attributes(ddb_hide=True)
     def hidden_calculated(self, obj):
         return obj
 
@@ -135,10 +135,21 @@ class SKUInline(admin.TabularInline):
         return "bob"
 
 
+@attributes(short_description="funky")
+def func(obj):
+    return f"f{obj.name}"
+
+
 @admin.register(models.Product)
 class ProductAdmin(AdminMixin, ProductMixin, admin.ModelAdmin):
     inlines = [SKUInline]
-    list_display = ["only_in_list_view", "annotated"]
+    list_display = [
+        "only_in_list_view",
+        "annotated",
+        func,
+        lambda o: f"l{o.name}",
+        "calculated_boolean",
+    ]
     ddb_hide_fields = ["hidden_model"]
     ddb_extra_fields = ["extra_model"]
 
@@ -149,3 +160,7 @@ class ProductAdmin(AdminMixin, ProductMixin, admin.ModelAdmin):
     @annotation
     def stealth_annotation(self, request, qs):
         return qs.annotate(stealth_annotation=F("name"))
+
+    @attributes(boolean=True)
+    def calculated_boolean(self, obj):
+        return obj.size
